#!/etc/opt/openit/perl -w
#
# $Id: psm.dm,v 1.17 2001/11/01 15:49:53 borgar Exp $
#
# (C) Copyright 2003 by Open Systems Computing AS, All rights reserved.
#
# This software may be used or copied only with the written permission of
# Open Systems Computing AS or in accordance with the terms and conditions
# stipulated in the agreement under which this software has been supplied.
#
# This file is part of Open iT Resources.
#
#HEAD##########################################################################
#
# @NAME:
#   ticks_set
# @AUTHOR:
#   Kjetil Yri
# @CREATED:
#   Thu Oct  2 12:51:32 MET 2003
# @DESCRIPTION:
#   Run test_cpu and set pacct clock ticks in openit.cfg.
#
#/HEAD#########################################################################

use strict;
use File::Copy;
use FileHandle;

my $pacct_ticks = undef;
my $ret = `/opt/openit/bin/test_cpu 2>/dev/null`;

if ($ret =~ m/failed/) {
    # Will also fail by default on batch installation
    print "WARNING: pacct test failed. Proceding anyway.\n";
} else {
    if( $ret =~ m/ok (\d+)$/ ) {
	$pacct_ticks = $1;
    }
}

my $config_file = "/etc/opt/openit/openit.cfg";

if ( ! insert_config_lines( $config_file, [], [ "PACCT_TICKS" ] )) {
    die "ERROR: Cannot set up Accounting directives in $config_file, $!\n";
}
if( defined $pacct_ticks ) {
    if ( ! insert_config_lines( $config_file,
				[ "PACCT_TICKS     $pacct_ticks\n" ], [] )) {
	die "ERROR: Cannot set up Accounting directives in $config_file, $!\n";
    }
}
sub insert_config_lines {
    my ( $filename, $lines_ref, $directives_ref ) = @_;

    # Dereference
    my @lines = @{$lines_ref};

    # initialize random generator
    srand( time() ^ ($$ + ($$ << 15)));
    # get random number for filename
    my $filename_postfix = rand(88998811);

    my $openit_tmp_file="/tmp/temporary_Open_iT_config_file.$filename_postfix";
    my $confhandle = new FileHandle();
    if ( defined( $directives_ref )) {
	&copy_file_and_remove_config_lines( $filename, $openit_tmp_file,
					    @{$directives_ref});
    } else {
	copy( $filename, $openit_tmp_file );
    }

    if (! $confhandle->open( ">>$openit_tmp_file" )) {
	die "Cannot append to file $openit_tmp_file\n";
    }

    my $ok = 1;

    foreach my $line ( @lines ) {
	$ok = $ok && print $confhandle $line;
    }

    $confhandle->close();
    copy( $openit_tmp_file, $filename );

    unlink ( $openit_tmp_file );

    return $ok;
}

sub copy_file_and_remove_config_lines {
    my ( $from_file_name, $to_file_name, @remove_configs ) = @_;

    open( INFILE, "< $from_file_name " ) || die "FATAL: Couldn't read from $from_file_name, $!";
    open( OUTFILE, "> $to_file_name ") || die "Couldn't write to $to_file_name, $!";

    my $line;
    while( defined( $line = <INFILE> ) ) {
	my $keep_this = 1;
	my $config_to_rm;
	foreach $config_to_rm ( @remove_configs ) {
	    if ( $line =~ /^\s*$config_to_rm/ ) {
		$keep_this = 0;
		last;
	    }
	}
	if ( $keep_this ) {
	    print OUTFILE $line;
	}
    }

    close INFILE;
    close OUTFILE;
}
